import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;


public class PersonTest {
	// method overloading
	public static void print(String name, int age) {
        System.out.printf("Person Name: %s, Age: %d\n", name, age);
	}
	// method overloading
	public static void print(Person person) {
        System.out.printf("Person Name: %s, Age: %d\n", person.getName(), person.getAge());
	}

     public static void main(String[] args) {
         
        System.out.println("People"); // people
        Person[] pArray = {
        		new Person("P", 30),
        		new Person("K", 20),
        		new Person("S", 10),
        		new Student("AB", 15, 2020),
        		new Staff("ACE", 25, "ACE")
        };
        for (Person p : pArray) {
        	System.out.println(p);
        }
        
        System.out.println("\nPeople sort by default (Person name)~~~~~");
        Arrays.sort(pArray, null);
        for (Person p : pArray) {
        	System.out.println(p);
        }
        
        System.out.println("\nPeople sort by (Person age)~~~~~");
        Arrays.sort(pArray, Person.AgeComparator);
        for (Person p : pArray) {
        	System.out.println(p);
        }
        
        Student[] sArray = {
        		new Student("P", 30, 2018),
        		new Student("K", 20, 2019),
        		new Student("S", 10, 1990),
        		new Student("AB", 15, 2020),
        };

        System.out.println("\nStudent sort by default (Person name)~~~~~");
        Arrays.sort(sArray, null); // Comparable interface
        for (Student s : sArray) {
        	System.out.println(s);
        }
        
        System.out.println("\nStudent sort by (Person age)~~~~~");
        Arrays.sort(sArray, Person.AgeComparator); // Comparator
        for (Student s : sArray) {
        	System.out.println(s);
        }
        
        System.out.println("\nStudent sort by (Student id)~~~~~");
        Arrays.sort(sArray, Student.IDComparator); // Comparator
        for (Student s : sArray) {
        	System.out.println(s);
        }
        
        System.out.println("\nStudent sort by (Student age)~~~~~");
        Arrays.sort(sArray, new Comparator<Student>() { // anonymous class Comparator
			@Override
			public int compare(Student o1, Student o2) {
				return o1.age - o2.age; // ascending order
			}     	
        });
        for (Student s : sArray) {
        	System.out.println(s);
        }
        
        System.out.println("\nStudent sort by (Student name)~~~~~");
        Arrays.sort(sArray, new Comparator<Student>() { // anonymous class Comparator
			@Override
			public int compare(Student o1, Student o2) {
				return o1.name.compareTo(o2.name); // ascending order
			}     	
        });
        for (Student s : sArray) {
        	System.out.println(s);
        }

        System.out.println("\nLambda Student sort by (Student age)~~~~~"); // lambda Comparator
        Arrays.sort(sArray, (Student o1, Student o2) -> o1.age - o2.age);
        for (Student s : sArray) {
        	System.out.println(s);
        }
        
        System.out.println("\nLambda Student sort by (Student name)~~~~~");
        Arrays.sort(sArray, (Student o1, Student o2) -> o1.name.compareTo(o2.name));
        for (Student s : sArray) {
        	System.out.println(s);
        }
        
        System.out.println("\nLambda Student sort by (Student id)~~~~~");
        Arrays.sort(sArray, (Student o1, Student o2) -> Integer.compare(o1.getId(), o2.getId()));
        for (Student s : sArray) {
        	System.out.println(s);
        }
        
        // Array -> ArrayList
        System.out.println("\n PersonList~~~~~");
        List<Person> pList = new ArrayList<>(Arrays.asList(pArray));
        for (Person p : pList) {
        	System.out.println(p);
        }
        
        System.out.println("\n PersonList sort by default (Person name)~~~~~");
        pList.sort(null);
        for (Person p : pList) {
        	System.out.println(p);
        }
        
        System.out.println("\n PersonList sort by (Person age)~~~~~");
        pList.sort(Person.AgeComparator);
        for (Person p : pList) {
        	System.out.println(p);
        }
   
        // HashMap
        System.out.println("\n PersonHashMap~~~~~");
        Map<Integer, Person> pMap = new HashMap<>();
        for (int i=0; i<pArray.length; i++) {
        	pMap.put(i, pArray[i]);
        }
        for (Map.Entry<Integer, Person> entry : pMap.entrySet()) {
        	System.out.println(entry.getKey() + " : " + entry.getValue());
        }

        System.out.println("\n PersonHashMap2~~~~~");
        Map<Order, Person> pMap2 = new HashMap<>();
        for (Order o : Order.values()) {
        	pMap2.put(o, pArray[o.ordinal()]);
        }
        for (Map.Entry<Order, Person> entry : pMap2.entrySet()) {
        	System.out.println(entry.getKey() + " : " + entry.getValue());
        }
        
        // pMap2 sort by Value
        System.out.println("\n PersonHashMap2 sort by Person name~~~~~");
        // entry list & sort
        List<Map.Entry<Order, Person>> entries = new ArrayList<>(pMap2.entrySet());
        entries.sort(new Comparator<Map.Entry<Order, Person>>() {
			@Override
			public int compare(Entry<Order, Person> o1, Entry<Order, Person> o2) {
				return o1.getValue().compareTo(o2.getValue());
			}       	
        });
        // sortedMap
        Map<Order, Person> sortedMap2 = new LinkedHashMap<>();
        for (Map.Entry<Order, Person> entry : entries) {
        	sortedMap2.put(entry.getKey(), entry.getValue());
        }
        pMap2 = sortedMap2;
        for (Map.Entry<Order, Person> entry : pMap2.entrySet()) {
        	System.out.println(entry.getKey() + " : " + entry.getValue());
        }
    }
}
